#include "cadef.h"
#include <epicsVersion.h>
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#if ((EPICS_VERSION==3 && EPICS_REVISION>=14) || EPICS_VERSION>3) 
#define EPICS_3_14 1
#endif /* version calculation */
extern int tineSetAlarm(char *chname,int sev,int status,float fval);

#ifdef EPICS_3_14 
static void printChidInfo(chid chid, char *message) {
  printf("\n%s\n",message);
  printf("pv: %s  type(%d) nelements(%ld) host(%s)",ca_name(chid),
	 ca_field_type(chid),ca_element_count(chid),ca_host_name(chid));
  printf(" read(%d) write(%d) state(%d)\n",
	 ca_read_access(chid),ca_write_access(chid),ca_state(chid));
}

static void exceptionCallback(struct exception_handler_args args) {
chid	chid = args.chid;
long	stat = args.stat; /* Channel access status code*/
const char  *channel;
static char *noname = "unknown";
  
  channel = (chid ? ca_name(chid) : noname);
  if(chid) printChidInfo(chid,"exceptionCallback");
  printf("exceptionCallback stat %s channel %s\n",ca_message(stat),channel);
}

static void connectionCallback(struct connection_handler_args args) {
chid	chid = args.chid;
   printChidInfo(chid,"connectionCallback");
}

static void accessRightsCallback(struct access_rights_handler_args args) {
chid	chid = args.chid;
   printChidInfo(chid,"accessRightsCallback");
}
#endif /* EPICS_3.14 */

static void eventCallback(struct event_handler_args eha) {
chid	chid = eha.chid;
static char value[256];
static char name[128];
char *ptr;
int severity=0;
int status=0;
float valAsFloat;
struct dbr_stsack_string * pStruct = (struct dbr_stsack_string *)eha.dbr;

   memset(value,0,256);
  if(eha.status!=ECA_NORMAL) {
    printChidInfo(chid,"eventCallbackBadStatus");
  } else {
    status=pStruct->status;
    severity=pStruct->severity;
    strncpy(value,pStruct->value,256); value[256] = 0;
    printf("Event Callback for '%s' = '%s'\n",ca_name(eha.chid),value);
    valAsFloat = atof(value);
    memset(name,0,128);
    strncpy(name,ca_name(eha.chid),128); name[128] = 0;
    if( (ptr=strstr(name,".VAL")) != NULL) *ptr=0;
    tineSetAlarm(name,severity,status,atof(value));
/* printf(".\n"); */
  }
}


#define DCA(c)   if(verboseTINE_CA) {c}
long verboseTINE_CA = 0;

int e2t_add_event(char *recName, int *pSev)
{
#ifdef EPICS_3_14 
  static int first = 1;
  chid mychid;
  evid myevid;
  if(first) {
    first=0;
    printf("set alarm for %s first\n", recName);
    SEVCHK(ca_context_create(ca_enable_preemptive_callback),"ca_cnt_create");
    SEVCHK(ca_add_exception_event(exceptionCallback,NULL),"ca_add_exception_event");
  }
  printf("set alarm for %s after first\n", recName);
  SEVCHK(ca_create_channel(recName,connectionCallback,pSev,20,&mychid),
	 "ca_create_channel");
  SEVCHK(ca_replace_access_rights_event(mychid,accessRightsCallback),
	 "ca_replace_access_rights_event");
  SEVCHK(ca_add_masked_array_event(DBR_STSACK_STRING,1,mychid,eventCallback,pSev,
	 (float) 0, (float) 0, (float) 0,&myevid,DBE_ALARM), "ca_add_event");

  printf("set alarm for %s -- last\n", recName);

#else /* EPICS_3.14 */
int status = 0;
chid *precID = NULL;
int len = 1;
evid *pevid;
DCA(printf("set alarm for %s\n", recName);)
  if(ca_search(recName, precID) != ECA_NORMAL) printf("ca_search failed\n");	
  if(ca_pend_io(0.5) != ECA_NORMAL) printf("ca_pend failed\n");
  if(ca_add_event(DBR_DOUBLE, *precID, eventCallback, pSev,pevid) != ECA_NORMAL) 
    printf("ca_add_event failed\n");
#endif /* EPICS_3.14 */
	return(1);
}

int e2t_pend_io()
{
  return(5);
}
