/* Example showing how to register a new command with iocsh */

#include <stdio.h>

#include <epicsExport.h>
#include <iocsh.h>

/* This is the command, which the vxWorks shell will call directly */
void e2tStart(const char *name) {
    if (name) {
      printf("E2tStart with param = '%s'\n", name);
    } else {
       puts("E2tStart with no param");
       /* E2T(); */ /* Albert */
       startE2T(); /* jan bernauer */
    }
}

/* Information needed by iocsh */
static const iocshArg     e2tStartArg0 = {"name", iocshArgString};
static const iocshArg    *e2tStartArgs[] = {&e2tStartArg0};
static const iocshFuncDef e2tStartFuncDef = {"e2tStart", 1, e2tStartArgs};

/* Wrapper called by iocsh, selects the argument types that e2tStart needs */
static void e2tStartCallFunc(const iocshArgBuf *args) {
    e2tStart(args[0].sval);
}

/* Registration routine, runs at startup */
static void e2tStartRegister(void) {
    iocshRegister(&e2tStartFuncDef, e2tStartCallFunc);
}
epicsExportRegistrar(e2tStartRegister);
