#ifndef _ALMLIB_H_
#define _ALMLIB_H_

#include "project.def"
#include "tinedefs.h"
#include "srvcore.h"
#include <time.h>

#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#define CASHOST_FILE "cashosts.csv"
#ifndef MAXALARMS
#define MAXALARMS 500
#endif
#define DEFAULT_ALARM_WINDOW 3
#define DEFAULT_ALARM_HEARTBEAT 1200
#define DEFAULT_INSTANT_WINDOW 4
#define DEFAULT_TERMINATION_WINDOW 4
#define DEFAULT_ALARM_SURVEY_INTERVAL 1000

#define almNEWALARM     0x01
#define almHEARTBEAT    0x02
#define almOSCILLATION  0x04
#define almDATACHANGE   0x08
#define almINSTANT      0x10
#define almTERMINATE    0x40
#define almTEST         0x80

#define ALARM_DATA_SHORTSIZE 6
#define ALARM_DATA_SIZE 64

#define ALARM_TAG_SIZE 32
#define ALARM_TAG_SHORTSIZE 16

#define ALARM_TEXT_SIZE 64
#define ALARM_TEXT_SHORTSIZE 40
#define ALARM_TEXT_LONGSIZE 128

typedef struct AlarmMsgStruct5
{
  char server[EXPORT_NAME_SHORTSIZE];
  char device[DEVICE_NAME_SHORTSIZE];
  char alarmTag[ALARM_TAG_SHORTSIZE];
  UINT32 alarmCode;
  UINT32 timestamp;
  UINT32 alarmMask;
  BYTE alarmData[ALARM_DATA_SHORTSIZE];
  BYTE alarmDataFormat;
  BYTE alarmDataArraySize;
  BYTE severity;
  BYTE descriptor;
  UINT16 alarmSystem;
} AMS5;                    /* legacy AMS sent out upon request */
#define AMS5_SIZE  72

typedef struct AlarmMsgStruct
{
  char server[EXPORT_NAME_SIZE];
  char device[DEVICE_NAME_SIZE];
  char alarmTag[ALARM_TAG_SIZE];
  UINT32 alarmCode;
  UINT32 timestamp;
  UINT32 timestampUSec;
  UINT32 starttime;
  UINT32 starttimeUSec;
  UINT32 alarmMask;
  BYTE alarmData[ALARM_DATA_SIZE];
  BYTE alarmDataFormat;
  BYTE alarmDataArraySize;
  BYTE severity;
  BYTE descriptor;
  UINT16 alarmSystem;
  BYTE reserved[2];
} AMS;                    /* AMS sent out upon request */
#define AMS_SIZE (EXPORT_NAME_SIZE+DEVICE_NAME_SIZE+ALARM_TAG_SIZE+6*4+ALARM_DATA_SIZE+2*4) /* 224 bytes */

typedef struct ADS5tag    /* Alarm Definition Structure */
{
   char alarmTag[ALARM_TAG_SHORTSIZE];
   UINT32 alarmCode;
   UINT32 alarmMask;
   short severity;
   BYTE alarmDataFormat;
   BYTE alarmDataArraySize;
   UINT32 timeRes;
   short hitsChannel;
   short hitsModule;
   char alarmText[ALARM_TEXT_SHORTSIZE];
   char deviceText[ALARM_TEXT_SHORTSIZE-2];
   UINT16 alarmSystem;  /* take 2 bytes from deviceText to hold the system ID */
   char dataText[ALARM_TEXT_SHORTSIZE];
   char url[ALARM_TEXT_SHORTSIZE];
   UINT32 ndefs;
} ADS5;
#define ADS5_SIZE  200

typedef struct ADStag    /* Alarm Definition Structure */
{
   char alarmTag[ALARM_TAG_SIZE];
   UINT32 alarmCode;  /* alarm code ID */
   UINT32 alarmMask;  /* alarm mask */
   UINT16 alarmSystem;/* alarm system ID */
   short  alarmSeverity;/* alarm severity */
   BYTE alarmDataFormat;
   BYTE alarmDataArraySize;
   BYTE reserved[2];
   char alarmText[ALARM_TEXT_SIZE];
   char deviceText[ALARM_TEXT_SIZE];
   char dataText[ALARM_TEXT_SIZE];
   char url[ALARM_TEXT_LONGSIZE];
} ADS;
#define ADS_SIZE  (ALARM_TAG_SIZE + 2*4 + 4*2 + 3*ALARM_TEXT_SIZE + ALARM_TEXT_LONGSIZE)  /* 368 */

typedef struct ADSListStruct
{
  ADS ads;
  struct ADSListStruct *next;
} ADSList;

typedef struct     /* Alarm Definition Structure + 64 char format string */
{
   char alarmTag[ALARM_TAG_SIZE];
   UINT32 alarmCode;  /* alarm code ID */
   UINT32 alarmMask;  /* alarm mask */
   UINT16 alarmSystem;/* alarm system ID */
   short  alarmSeverity;/* alarm severity */
   BYTE alarmDataFormat;
   BYTE alarmDataArraySize;
   BYTE reserved[2];
   char alarmText[ALARM_TEXT_SIZE];
   char deviceText[ALARM_TEXT_SIZE];
   char dataText[ALARM_TEXT_SIZE];
   char url[ALARM_TEXT_LONGSIZE];
   char alarmDataFormatString[64]; /* for easy reading by csv and xml parsers */
} ADSIS;

typedef struct AMRBtag
{
  UINT32 almStartTime;
  UINT32 almStopTime;
  UINT32 almMinSeverity;
  UINT32 reserved;
} AMRB;                   /* Alarm Message Request Block */
#define AMRB_SIZE  16

typedef struct ALARMstruct/* local alarm list          */
{
  UINT32 timestamp;       /* data timestamp */
  UINT32 timestampUSec;   /* usec fraction of data timestamp */
  UINT32 starttime;       /* initial data timestamp */
  UINT32 starttimeUSec;   /* usec fraction of initial data timestamp */
  UINT32 alarmCode;       /* standardized return code (implies severity)   */
  int    removalBlocked;  /* flag to keep alarm in table for CAS readout   */
  BYTE   alarmData[ALARM_DATA_SIZE]; /* alarm specific data (database interpreted)    */
  short  alarmSeverity;   /* dynamic override of static alarm severity */
  BYTE   alarmStatus;     /* corresponds to the descriptor                 */
  BYTE   clrCount;        /* record of clear requests                      */
  struct ALARMstruct *next;
} ALARM;

#define AssignAlmTimestamp(a,t) {\
  (a)->timestamp = (UINT32)(t);\
  (a)->timestampUSec = (((UINT32)((t) * 1000000)) % 1000000);\
}

#define ALMARM_WATCH_FILE "almwatch.csv"
typedef struct AlarmWatchStruct
{
  char eqm[EQM_NAME_SIZE];
  char dev[DEVICE_NAME_SIZE];
  char prp[PROPERTY_NAME_SIZE];
  int  siz;
  int  fmt;
  int  atyp;              /* array type */
  int  asys;              /* alarm system id */
  int  sev;               /* general highest severity */
  int  hisev;             /* high severity */
  int  hiwarnsev;         /* high warning severity */
  int  losev;             /* low severity */
  int  lowarnsev;         /* low warning severity */
  int  chkNormal;         /* TRUE => check vs normal; FALSE = check threshold */
  UINT32 mask;            /* if fmt is an integer type */
  UINT32 normal;          /* normal non-alarm state value (after mask) */ 
  int  cnt;
  int  cntThreshold;      /* set alarm when counter > threshold */
  float hi;               /* high threshold */
  float lo;               /* low threshold */
  float hiwarn;           /* high warn threshold */
  float lowarn;           /* low warn threshold */
} AWS;
#define AWS_SIZE (EQM_NAME_SIZE+DEVICE_NAME_SIZE+PROPERTY_NAME_SIZE+18*4)

typedef struct AWSListStruct
{
  AWS aws;
  struct AWSListStruct *nxt;
} AWSList;

extern AWSList *almWatchList;
extern int nAlmWatchList;

typedef struct     /* Alarm Watch Structure + 64 char format string */
{
  char eqm[EQM_NAME_SIZE];
  char dev[DEVICE_NAME_SIZE];
  char prp[PROPERTY_NAME_SIZE];
  int  siz;
  int  fmt;
  int  atyp;              /* array type */
  int  asys;              /* alarm system id */
  int  sev;               /* general highest severity */
  int  hisev;             /* high severity */
  int  hiwarnsev;         /* high warning severity */
  int  losev;             /* low severity */
  int  lowarnsev;         /* low warning severity */
  int  chkNormal;         /* TRUE => check vs normal; FALSE = check threshold */
  UINT32 mask;            /* if fmt is an integer type */
  UINT32 normal;          /* normal non-alarm state value (after mask) */ 
  int  cnt;
  int  cntThreshold;      /* set alarm when counter > threshold */
  float hi;               /* high threshold */
  float lo;               /* low threshold */
  float hiwarn;           /* high warn threshold */
  float lowarn;           /* low warn threshold */
  char alarmDataFormatString[64]; /* for easy reading by csv and xml parsers */
} AWSIS;

/**
 * \struct ALM_THRESHOLDS
 * \brief Defines a TINE Alarm Watch Threshold object
 *
 * - hi is the high value threshold used for setting value_too_high alarms
 * - hiwarn is the high warning value threshold used for setting warn_too_high alarms
 * - lo is the low value threshold used for setting value_too_low alarms
 * - lowarn is the low warning value threshold used for setting warn_too_low alarms
 */
typedef struct 
{
  int cnt;       /* number successive threshold exceeded before applying alarm */
  int hisev;     /* high severity */
  int hiwarnsev; /* high warning severity */
  int losev;     /* low severity */
  int lowarnsev; /* low warning severity */
  float hi;      /* high threshold */
  float hiwarn;  /* high threshold warning (approaching high threshold) */
  float lo;      /* low threshold */
  float lowarn;  /* low threshold warning (approaching low threshold) */
  UINT32 mask;   /* if non-zero -> apply mask and check vs. normal value */
  UINT32 normal; /* normal valid data value */
} ALM_THRESHOLDS;

extern ALARM *almList;

/* appendAlarmWatchTable() */
/**
 * \brief Inserts a property to be monitored into the local alarm server's Watch Table
 * 
 * Certain alarms are to be set whenever the value of a property exceeds a defineable 
 * threshold.  Such alarms can be managed automatically by the local alarm server if the
 * alarm criteria are entered into the alarm watch table.  This can be achieved by calling
 * this routine (or supplying a startup configuration file almwatch.csv).
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param prp is the property which is to be called by the local alarm server
 * \param dev is the device name associated with the property to be called by the local alarm server
 * \param siz is the data array size to be called by the local alarm server
 * \param fmt is the TINE data format to be called by the local alarm server
 * \param atyp is the TINE data array type to be applied to the property called by the local alarm server
 * \param sev is the severity of the alarm issued when the data returned by the call exceed the
 *  given thresholds.
 * \param sys is the alarm system identifier to be associated with the alarm.
 * \param thr is an ALM_THRESHOLDS object specifying the threshold criteria for setting
 *  the alarm
 *  
 * \note Information can also be entered into the alarm watch table by supplying the startup
 * configuration file almwatch.csv, where the input parameters are given in the relevant columns.
 * This is frequently the preferred way of inputting such alarm information, since if does not
 * involve hard-coding such alarm criteria.
 *
 * \return 0 upon success, otherwise a TINE error code.
 *
 */
TINE_EXPORT int AppendAlarmWatchTable(char *eqm,char *prp,char *dev,int siz,int fmt,int atyp, int sev,int sys,ALM_THRESHOLDS *thr);
#define appendAlarmWatchTable(a1,a2,a3,a4,a5,a6,a7,a8,a9) AppendAlarmWatchTable(a1,a2,a3,a4,a5,a6,a7,a8,a9)

/* MaxNumAlarmDefinitions */
/**
 * \brief Determines the maximum alarm definition table size when using the appendAlarmInfoTable()
 * API call to register alarm table information.
 * 
 * If an alarm definition configuration file \<local name\>-alarms.csv is used, this variable
 * plays no role.  When the API call appendAlarmInfoTable() is used to register alarm table
 * information, this variable is used to pre-allocate the appropriate alarm definition table
 * size.
 *
 * \b Default: 100
 *  
 * \b Example:

  MaxNumAlarmDefinitions = 500;  

 */
extern TINE_EXPORT int MaxNumAlarmDefinitions;
/* appendAlarmInfoTable() */
/**
 * \brief Inserts an alarm definition into the alarm definition table.
 * 
 * As an alternative to the \<local name\>-alarms.csv configuration file,
 * the front end server can make use of this API call in order to fill in
 * the alarm definition table describing locally generated alarms.
 * This is particularly useful for embedded platforms where there is no 
 * file system, or where a TINE server is used as a translation layer and 
 * needs to map a given alarm system onto the TINE alarm system.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param ads is a pointer to an Alarm Definition Structure (ADS) containing the
 * alarm table information which is to be appended to the alarm definition table.
 *  
 * \return 0 upon success, otherwise a TINE error code.
 *
 * \include eg_appendAlarmInfoTable.c
 * 
 */
TINE_EXPORT int AppendAlarmInfoTable(char *eqm,ADS *ads);

/* SetDeviceAlarm() */
/**
 * \brief Inserts an alarm into the local alarm server table.
 * 
 * Analogous to SetAlarmEx(), this call inserts alarms into the local alarm server table.
 * A server can set an alarm for a specific device by giving its device name.  This is sometime
 * preferred over specifying its device number as in SetAlarm() or SetAlarmEx().
 * For further information, please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devname is the registered device name of the device for which the alarm is to be set.
 * \param almCode is the registered alarm code which defines the alarm
 * \param almData is (6-byte) the optional alarm data set, which supplies additional information for the alarm.
 * \param almFlags is the optional alarm flag which is used in the alarm descriptor information.
 *  
 * \return 0 upon success, otherwise a TINE error code.
 *
 * \include eg_SetDeviceAlarm.c
 */
TINE_EXPORT int SetDeviceAlarm(char *eqm,char *devname,UINT32 almCode,BYTE *almData,BYTE almFlags);

/* SetAlarmsEx() */
/**
 * \brief Inserts an alarm into the local alarm server table.
 * 
 * This is the extended call corresponding to SetAlarm(), the difference being 
 * the additional argument flags which can be used to set the alarm descriptor.
 * A server can set an alarm for a specific device by giving its device number.  
 * For further information, please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devNr is the device number of the device for which the alarm is to be set.
 * \param almCode is the registered alarm code which defines the alarm
 * \param almData is (6-byte) the optional alarm data set, which supplies additional information for the alarm.
 * \param almFlags is the optional alarm flag which is used in the alarm descriptor information.
 *  
 * \return 0 upon success, otherwise a TINE error code.
 *
 * \include eg_setAlarmEx.c
 */
TINE_EXPORT int SetAlarmEx(char *eqm,short devNr,UINT32 almCode,BYTE *almData,BYTE almFlags);

/* SetAlarm() */
/**
 * \brief Inserts an alarm into the local alarm server table.
 * 
 * A server can set an alarm for a specific device by giving its device number.  
 * For further information, please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devNr is the device number of the device for which the alarm is to be set.
 * \param almCode is the registered alarm code which defines the alarm
 * \param almData is (6-byte) the optional alarm data set, which supplies additional information for the alarm.
 *  
 * \return 0 upon success, otherwise a TINE error code.
 *
 * \include eg_SetAlarm.c
 */
TINE_EXPORT int SetAlarm(char *eqm,short devNr,UINT32 almCode,BYTE *almData);

TINE_EXPORT int getAlarmsEx(char *eqmName,AMS *ams,short num,short minSeverity,time_t starttime,time_t stoptime,short startnum);
TINE_EXPORT int getAlarms(char *tag,AMS *ams,short num,short minSeverity);
int numAlarmsEx(char *eqmName,UINT32 *almTimeStamp,short *almSeverity,short *NrAtAlmTimeStamp,short *NrAtAlmSeverity);
int numAlarms(char *tag,UINT32 *almTimeStamp,short *almSeverity);

/* ClearAlarm() */
/**
 * \brief Instructs the local alarm server table that the given alarm is to be cleared.
 * 
 * A server can clear alarms for specific device by giving its device number.  
 * Note that all alarms for the device specified are cleared.  Also note, 
 * that "clearing" an alarm means that its "clear" counter is incremented.  
 * It is not removed immediately from the local alarm list. For further information, 
 * please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devNr is the device number of the device for which all alarms is to be cleared.
 *  
 * \include eg_ClearAlarm.c
 */
TINE_EXPORT void ClearAlarm(char *eqm,short devNr);
/* ClearDeviceAlarm() */
/**
 * \brief Instructs the local alarm server table that the given alarm is to be cleared.
 * 
 * A server can clear alarms for specific device by giving its registered device name.  
 * Note that all alarms for the device specified are cleared.  Also note, 
 * that "clearing" an alarm means that its "clear" counter is incremented.  
 * It is not removed immediately from the local alarm list. For further information, 
 * please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devname is the registered device name of the device for which all alarms is to be cleared.
 *  
 * \include eg_ClearDeviceAlarm.c
 */
TINE_EXPORT void ClearDeviceAlarm(char *eqm,char *devname);

/* RemoveDeviceAlarm() */
/**
 * \brief Instructs the local alarm server table that the given alarm is to be marked for removal.
 * 
 * A server can remove alarms for specific device by giving its registered device name.  
 * Note that only the alarm specified for the device specified is removed.  Also 
 * note, that "removing" an alarm means that termination bit will be set.  It is 
 * subsequently marked for removal from the local alarm list.  For further information, 
 * please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devname is the registered device name of the device for which all alarms is to be removed.
 * \param code is the alarm code, which identifies the specific alarm to be removed
 *
 * \include eg_RemoveDeviceAlarm.c
 */ 
TINE_EXPORT void RemoveDeviceAlarm(char *eqm,char *devname,UINT32 code);

/* RemoveAlarm() */
/**
 * \brief Instructs the local alarm server table that the given alarm is to be marked for removal.
 * 
 * A server can remove alarms for specific device by giving its device number.  
 * Note that only the alarm specified for the device specified is removed.  Also 
 * note, that "removing" an alarm means that termination bit will be set.  It is 
 * subsequently marked for removal from the local alarm list.  For further information, 
 * please see the discussion of the local alarm server.
 *
 * \param eqm is the 6-character local equipment identifier name, which is internal to server.
 * \param devNr is the device number of the device for which all alarms is to be removed.
 * \param code is the alarm code, which identifies the specific alarm to be removed
 *
 * \include eg_RemoveAlarm.c
 */ 
TINE_EXPORT void RemoveAlarm(char *eqm,short devNr,UINT32 code);
short findSeverity(char *name,UINT32 cc);
int RegisterSeverity(int almCode,int almSeverity);
int fillHRTstamp(BYTE *HRTstamp);
void getAlarmInfo(void);
int getAlarmDefs(char *eqm,ADSList **adslst);
int numAlarmDefs(char *eqm);
ADS *getAlarmTable(char *eqm,UINT32 code);
int getAlarmInfoTable(char *eqm);
int initAlarmServer(void);
TINE_EXPORT int GetAlarm(char *eqmName,char *devname,AMS *ams);
int SetAlarmByDeviceNumber(char *name,short devNr,UINT32 almCode,BYTE *almData,BYTE flags,double ts);
int SetHrtAlarm(char *name,short devNr,UINT32 almCode,BYTE *almData,BYTE flags,time_t t);
int isCASInUserList(char *eqm);
int prepOutgoingADS(BYTE *adsstream,ADSList *adslst,char *stag,int length);
int prepOutgoingAlarmData(BYTE *datastream,int size,short fmt);
int prepOutgoingAMS(BYTE *amsstream,AMS *amslst,char *stag,int length);
int prepIncomingAMRB(AMRB *amrb,BYTE *amrbstream);
int prepOutgoingAWS(BYTE *awsstream,AWSList *awslst,char *stag,int length);
int prepIncomingAWS(BYTE *awsstream,AWS *aws,char *stag,int length);
void checkAlarmsInWatchTable(void);

extern char *tagList[];
extern int numStoredAlarms;

/* AlarmWindow */
/**
 * \brief Determines the minimum time an alarm is to remain in the local alarm table 
 * before it can be removed.
 * 
 * IO routines can sometimes set and clear alarms at such a high frequency that 
 * an alarm could go unnoticed if it appeared and disappeared before a client had 
 * a chance to read it.  The AlarmWindow value determines the amount of time (in seconds) 
 * an alarm must remain in the local alarm list before it can be removed.  The default 
 * is 3 seconds, which is generally fine for a central alarm reader polling at 1 second 
 * intervals.
 *
 * \b Default: 3 sec
 *  
 * \b Example:

  AlarmWindow = 10;  // keep alarms in the table a bit longer

 */
extern TINE_EXPORT int AlarmWindow;

/* AlarmHeartbeat */
/**
 * \brief Determines the time interval at the 'HEARTBEAT' flag will be re-issued 
 * for persistent alarms.
 * 
 * The purpose of an alarm is to alert operators or maintenance staff to a problem.  
 * If an alarm persists and is not dealt with, its timestamp is adjusted to the current 
 * time at intervals given by the AlarmHeartbeat value.  This will 
 * call attention to the alarm.  It also places the alarm at regular intervals in an 
 * archive of alarms.  The developer can set this to any desired value (usually at 
 * initialization time).
 *
 * \b Default: 1200 sec
 *  
 * \b Example:

  AlarmHeartbeat = 3600;  // heartbeat once per hour

 */
extern TINE_EXPORT int AlarmHeartbeat;

/* AlarmInstantWindow */
/**
 * \brief Determines the minimum time an alarm is to remain in the local alarm table 
 * before it can be removed.
 * 
 * Transient alarms (with no duration) are immediately marked for termination as soon 
 * as they are set.  The AlarmInstantWindow determines the amount of time in seconds 
 * an "Instant" Alarm is to remain in the alarm list (default 4 seconds).
 *
 * \b Default: 4 sec
 *  
 * \b Example:

  AlarmInstantWindow = 10;  // keep transient alarms in the table a bit longer

 */
extern TINE_EXPORT int AlarmInstantWindow;

/* TerminationWindow */
/**
 * \brief Determines the minimum time an expired alarm is to remain in the local alarm table. 
 * 
 * Terminated alarms remain in the local alarm table for an amount of time specified by
 * this variable.  Thus a client monitoring the local alarm list will be able tosee that 
 * the alarm was set and has finished.
 *
 * \b Default: 4 sec
 *  
 * \b Example:

  TerminationWindow = 10;  // keep terminated alarms in the table a bit longer

 */
extern TINE_EXPORT int TerminationWindow;
extern int NTAGS;
extern int NSEVERITYCODE;

/* MaxNumberAlarms */
/**
 * \brief Determines the maximum number of alarms which can be maintained in the
 * local alarm server's alarm list.
 * 
 * Each registered device can have an associated alarm link list.  The length of this 
 * list will not exceed the value entered for MaxNumAlarms (default 100).  The default 
 * is generally more than enough.  Primarily this variable serves as a safety valve 
 * against sudden alarm avalanches.
 *
 * \b Default: 100
 *  
 * \b Example:

  MaxNumberAlarms = 500;  

 */
extern TINE_EXPORT int MaxNumberAlarms;

/* AlarmTestState */
/**
 * \brief Determines the minimum time an alarm is to remain in the local alarm table 
 * before it can be removed.
 * 
 * When developing the alarm logic for a particular server is frequently useful to 
 * mark the alarms being set with the almTEST bit.  This can either be done on an 
 * individual basis using the SetAlarmEx() call, or one can set the global variable 
 * AlarmTestState to a non-zero value.
 *
 * \b Default: FALSE
 *  
 * \b Example:

  AlarmTestState = TRUE;  // just testing ...

 */
extern int AlarmTestState;
extern int ALMdebug;
extern int gRequireReadFromCAS;

extern int gIsCentralAlarmServer;
TINE_EXPORT void isCentralAlarmServer(int value);
TINE_EXPORT int SetAlarmSurveyInterval(UINT32 interval);
TINE_EXPORT int GetAlarmSurveyInterval(void);

/* SetAlarmTableMaximumSize() */
/**
 * \brief Sets the maximum number of alarms managed by the local alarm server
 * 
 * A server keeps a maximum number of alarms in its local table at one time
 * before it begins to discard the older alarms.  The caller can adjust this number
 * with this call. Default is 500.
 *
 * \param tableSize is the maximum number of alarms to be maintained in the local alarm table.
 *
 */ 
TINE_EXPORT void SetAlarmTableMaximumSize(int tableSize);

/* GetAlarmTableMaximumSize() */
/**
 * \brief Gets the maximum number of alarms managed by the local alarm server
 * 
 * A server keeps a maximum number of alarms in its local table at one time
 * before it begins to discard the older alarms.  The caller can retrieve this number
 * with this call. Default is 500.
 *
 * \return the maximum number of alarms to be maintained in the local alarm table.
 *
 */ 
TINE_EXPORT int GetAlarmTableMaximumSize(void);

/* SetAlarmTerminationWindow() */
/**
 * \brief Sets the size of the termination window in seconds before a terminated alarm is
 * discarded from the local alarm list.
 * 
 * A server keeps keeps a terminated alarm in its list (marked as terminated) for a 
 * length of time specified by the termination window.  This allows the central alarm server
 * and/or other interested parties enough time to discover that the alarm has been terminated.
 * Default is 4 seconds.  The caller can set this window with this call.
 *
 * \param timeInSeconds is the size of the termination window.
 *
 */ 
TINE_EXPORT void SetAlarmTerminationWindow(int timeInSeconds);

/* GetAlarmTerminationWindow() */
/**
 * \brief Gets the size of the termination window in seconds before a terminated alarm is
 * discarded from the local alarm list.
 * 
 * A server keeps keeps a terminated alarm in its list (marked as terminated) for a 
 * length of time specified by the termination window.  This allows the central alarm server
 * and/or other interested parties enough time to discover that the alarm has been terminated.
 * Default is 4 seconds.  The caller can discover this time window with this call.
 *
 * \param timeInSeconds is the size of the termination window.
 *
 */ 
TINE_EXPORT int GetAlarmTerminationWindow(void);

/* SetAlarmTerminationWindow() */
/**
 * \brief Sets the size of the new alarm window in seconds before a new alarm is
 * no longer declared as new.
 * 
 * A server flags an alarm as new when it first appears in its list.
 * After a short time span, the 'new' flag is removed by additional calls to SetAlarm for the
 * same alarm.
 * Default is 3 seconds.  The caller can set this window with this call.
 *
 * \param timeInSeconds is the size of the new alarm window.
 *
 */ 
TINE_EXPORT void SetNewAlarmWindow(int timeInSeconds);

/* GetNewAlarmWindow() */
/**
 * \brief Sets the size of the new alarm window in seconds before a new alarm is
 * no longer declared as new.
 * 
 * A server flags an alarm as new when it first appears in its list.
 * After a short time span, the 'new' flag is removed by additional calls to SetAlarm for the
 * same alarm.
 * Default is 3 seconds.  The caller can discover this time window with this call.
 *
 * \param timeInSeconds is the size of the new alarm window.
 *
 */ 
TINE_EXPORT int GetNewAlarmWindow(void);

/* SetAlarmHeartbeat() */
/**
 * \brief Sets the size of the heartbeat alarm window in seconds before the 'heartbeat' flag is
 * applied to an alarm.
 * 
 * A server flags an alarm with a heartbeat when it has persistent in the local alarm list for 
 * a time span equal to this window.
 * Default is 1200 seconds.  The caller can set this window with this call.
 *
 * \param timeInSeconds is the size of the heartbeat window.
 *
 */ 
TINE_EXPORT void SetAlarmHeartbeat(int timeInSeconds);

/* GetAlarmHeartbeat() */
/**
 * \brief Gets the size of the heartbeat alarm window in seconds before the 'heartbeat' flag is
 * applied to an alarm.
 * 
 * A server flags an alarm with a heartbeat when it has persistent in the local alarm list for 
 * a time span equal to this window.
 * Default is 1200 seconds.  The caller can discover this time window with this call.
 *
 * \param timeInSeconds is the size of the heartbeat window.
 *
 */ 
TINE_EXPORT int GetAlarmHeartbeat(void);

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* _ALMLIB_H_ */
